# -*- coding: utf-8 -*-
"""
Enables the user to add style plugin that displays a html tag with
the provided settings from the style plugin.
"""
from __future__ import unicode_literals

import os
from django.db import models
from django.conf import settings
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext, ugettext_lazy as _
from djangocms_attributes_field.fields import AttributesField

from cms.models import CMSPlugin


class Attribute(AttributesField):
    def validate_key(self, key):
        pass

@python_2_unicode_compatible
class CustomElement(CMSPlugin):
    """
    Renders a given ``TAG_CHOICES`` element with additional attributes
    """
    class Meta:
        abstract = True
        
    tag_type = 'custom-element'
    template = 'custom-element.html'

    label = models.CharField(
        verbose_name=_('Label'),
        blank=True,
        max_length=255,
        help_text=_('Overrides the display name in the structure mode.'),
    )
    classes = models.CharField(
        verbose_name=_('Additional classes'),
        blank=True,
        max_length=255,
        help_text=_('Additional comma separated list of classes '
            'to be added to the element e.g. "row, column-12, clearfix".'),
    )
    id_name = models.CharField(
        verbose_name=_('ID name'),
        blank=True,
        max_length=255,
    )
    attributes = Attribute(
        verbose_name=_('Attributes'),
        blank=True,
        excluded_keys=['class', 'id', 'style'],
    )
    js = models.TextField(null=True, blank=True)
    css = models.TextField(null=True, blank=True)
    
    def __str__(self):
        return self.label


class GenericCustomElement(CustomElement):

    tag_type = models.CharField(verbose_name=_('Tag type'), max_length=255)
    template = models.CharField(verbose_name=_('Template'), max_length=255, default='generic-custom-element.html')

    js_asset = models.FileField(verbose_name=_('JS asset'), null=True, blank=True, upload_to=os.path.join('custom_elements', 'js_assets'))
    css_asset = models.FileField(verbose_name=_('CSS asset'), null=True, blank=True, upload_to=os.path.join('custom_elements', 'css_assets'))
    
    async_load = models.BooleanField(default=False)

    class Meta:
        verbose_name = _('Generic CustomElement')
        verbose_name_plural = _('Generic CustomElements')

"""
class ExampleCustomElement(CustomElement):

    class Meta:
        verbose_name = _('Example CustomElement')
        verbose_name_plural = _('Example CustomElements')

    tag_type = 'example-element'
    template = 'example-element.html'
"""



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase

from .models import GenericCustomElement


class CustomElementPlugin(CMSPluginBase):
    name = _('Custom Element')
    
    fieldsets = (
        (None, {
            'fields': (
                'label', 'id_name',
                ('classes', 'attributes'),
                ('js', 'css'),
            ),
        }),
    )

    def get_render_template(self, context, instance, placeholder):
        return 'custom_elements/elements/{}'.format(instance.template)



class GenericCustomElementPlugin(CustomElementPlugin):
    model = GenericCustomElement
    name = _('Generic custom element')
        
    fieldsets = (
        (None, {
            'fields': (
                'tag_type',
                'label', 'id_name',
                ('classes', 'attributes'),
                ('js_asset', 'css_asset'),
                ('js', 'css'),
                'async_load',
            ),
        }),
    )
    
    allow_children = True

plugin_pool.register_plugin(GenericCustomElementPlugin)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-02-14 14:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangocms_pai_custom_elements.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0020_old_tree_cleanup'),
    ]

    operations = [
        migrations.CreateModel(
            name='GenericCustomElement',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_pai_custom_elements_genericcustomelement', serialize=False, to='cms.CMSPlugin')),
                ('label', models.CharField(blank=True, help_text='Overrides the display name in the structure mode.', max_length=255, verbose_name='Label')),
                ('classes', models.CharField(blank=True, help_text='Additional comma separated list of classes to be added to the element e.g. "row, column-12, clearfix".', max_length=255, verbose_name='Additional classes')),
                ('id_name', models.CharField(blank=True, max_length=255, verbose_name='ID name')),
                ('attributes', djangocms_pai_custom_elements.models.Attribute(blank=True, default=dict, verbose_name='Attributes')),
                ('js', models.TextField(blank=True, null=True)),
                ('css', models.TextField(blank=True, null=True)),
                ('tag_type', models.CharField(max_length=255, verbose_name='Tag type')),
                ('template', models.CharField(default='generic-custom-element.html', max_length=255, verbose_name='Template')),
                ('js_asset', models.FileField(blank=True, null=True, upload_to='custom_elements/js_assets', verbose_name='JS asset')),
                ('css_asset', models.FileField(blank=True, null=True, upload_to='custom_elements/css_assets', verbose_name='CSS asset')),
                ('async_load', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name_plural': 'Generic CustomElements',
                'verbose_name': 'Generic CustomElement',
            },
            bases=('cms.cmsplugin',),
        ),
    ]
